#!/bin/sh

###############################################################
#   This script should be hooked when invoking SYNOShareSet()
#   Usable environment variable:
#       ORIGIN_SHARE_NAME, SHARE_NAME, ORIGIN_SHARE_PATH, SHARE_PATH,
#       ORIGIN_SHARE_STATUS, SHARE_STATUS
#
#   Note: The script should bear to be invoked multiple times 
###############################################################

#Include common scripts
. /var/packages/CloudStation/target/scripts/CloudStationCommon


SHARE_SET_ACT="ShareSet"

CheckIfHook()
{	
	if [ "x" = "x$ORIGIN_SHARE_PATH" ]; then
		#For create case
		return 0;
	fi

	if [ "x$ORIGIN_SHARE_PATH" = "x$SHARE_PATH" ]; then
		#In the same volume
		return 0;
	fi
	
	return 1;
}

is_homes_check()
{
	if [ "x" = "x${ORIGIN_SHARE_NAME}" ]; then
		return 0;
	fi

	if [ "xhomes" != "x${ORIGIN_SHARE_NAME}" ]; then
		return 0;
	fi

	return 1;
}

cstn_share_repo_rename()
{
	local vid=`get_share_vid homes`
	local home_vol="/volume${vid}"
	local share_repo_path="$home_vol/@cloudstation/@cstn-share"
	
	local from_path="${share_repo_path}/@$1"
	local to_path="${share_repo_path}/@$2"

	logger -p 0 "rename share from $from_path to $to_path"

	if [ ! -d "${from_path}" ]; then
		logger -p 0 "$from_path does not exist"
			return;
		fi

	touch "/tmp/sf_repo_mv"
	
	#Transfer cstn privilege from old to new user
	rm -rf "$to_path" > /dev/null 2>&1
	mv -f "${from_path}" "${to_path}" > /dev/null 2>&1
	
	
	rm -rf "/tmp/sf_repo_mv"
}

cstn_home_change_vol()
{
	
	local from_loc="$1";
	local to_loc="$2";

	if [ ! -d "${from_loc}/@cloudstation" ]; then
		logger -p 0 "$from_loc/@cloudstation does not exist"
			return;
		fi
	

	touch "/tmp/sf_repo_mv"
	
		#In different volumes
			rm -rf "${to_loc}/@cloudstation" > /dev/null 2>&1
			mv "${from_loc}/@cloudstation" "${to_loc}/@cloudstation"  > /dev/null 2>&1
	
	rm -rf "/tmp/sf_repo_mv"
	
	logger -p 0 "move homes from $from_loc/@cloudstation to $to_loc/@cloudstation successfully"
	
	set_db_vol "$CSTN_DB_PATH_DIR/$CSTN_DB_CONF" db-vol "${to_loc}"
}

#At begining, acquire package settings
PackageInfoGet;

case $1 in
    --sdk-mod-ver)
		#Print SDK support version
		echo "2.0";
    	;;
    --name)
		#Print package name
		echo ${CSTN_PKG_NAME};
    	;;
    --pkg-ver)
		#Print package version
		echo ${CSTN_PKG_VERSION};
    	;;
    --vendor)
		#Print package vendor
		echo ${CSTN_PKG_VENDOR};
    	;;
    --pre)
    
    	is_cstn_upgrade_process_done;    	
    	if [ 0 -eq $? ]; then
			exit 0;
        fi
    
		PackageEnableGet;
		if [ 0 -eq $? ]; then
			exit 0
		fi

		CheckIfHook;
		if [ 0 -eq $? ]; then
			return;
		fi
		
		is_homes_check;
		if [ 1 -eq $? ]; then
			${CSTN_START_SCRIPT} stop > /dev/null 2>&1;
		else
			${CSTN_USRCTL_BIN} -s 0 "@${ORIGIN_SHARE_NAME}" 0 0		
		fi
    ;;
    --post)
    	
    	is_cstn_upgrade_process_done;    	
    	if [ 0 -eq $? ]; then
			exit 0;
        fi
    
		PackageEnableGet;
        is_pkg_enable=$?;
		is_homes=0;

		#Actions before share set
		ori_share="${ORIGIN_SHARE_NAME}";
		new_share="${SHARE_NAME}";
	
		CheckIfHook;
		if [ 0 -eq $? ]; then
			touch "${CSTN_RELOAD_USERDB_CACHE_MARK}";
			return;
		fi

		is_homes_check;
		if [ 1 -eq $? ]; then
			is_homes=1;
		fi
	
		if [ "0" != "$RESULT" ]; then
			ErrMsg "$SHARE_SET_ACT" "Try to start cstn for bad result..."
			if [ 1 -eq $is_pkg_enable ]; then
				if [ 1 -eq $is_homes ]; then
					${CSTN_START_SCRIPT} start > /dev/null 2>&1;
				else
					${CSTN_USRCTL_BIN} -s 0 "@${SHARE_NAME}" 1 0
				fi
			fi
			exit 0;
		fi
		# old:
		#1. homes to not homes, do nothing
		#2. not homes to homes, do nothing
		#3. homes to homes but change volume
		#
		# new share move :
		# 1. share rename, (change vol or not): rename $db-vol/@cstn-share/@old to $db-vol/@cstn-share/@new
		#	1.1 update user_db
		# 2. share change vol: do nothing
		# 3. homes can't rename
		# 4. homes change volume: do nothing
		local ori_vol=`VolPathGet "${ORIGIN_SHARE_PATH}"`
		local new_vol=`VolPathGet "${SHARE_PATH}"`
		local shared_rename=0
		local vol_chaned=0

		# if change volume, only homes need to be moved
		if [ "x$ori_share" = "x$new_share" ]; then
			logger -p 0 "volume_changed: from $ori_vol to $new_vol"
			vol_changed=1
		else
			logger -p 0 "share_rename: from $ori_share to $new_share"
			share_rename=1
			fi
		
		if [ $share_rename -eq 1 ]; then
			if [ 1 -eq $is_homes ]; then
				ErrMsg "$SHARE_SET_ACT" "homes is changed to another name!"
				exit 0;
			else
				cstn_share_repo_rename "${ori_share}" "${new_share}"
				${CSTN_USRCTL_BIN} -r "@${ori_share}" "@${new_share}" > /dev/null 2>&1
				${CSTN_USRCTL_BIN} -s 0 "@${new_share}" 1 0 > /dev/null 2>&1
			fi
		fi
		
		if [ $vol_changed -eq 1 ]; then
			if [ 1 -eq $is_homes ]; then
				cstn_home_change_vol "${ori_vol}" "${new_vol}"
			else
				${CSTN_USRCTL_BIN} -s 0 "@${new_share}" 1 0 > /dev/null 2>&1
			fi
		fi
		
		if [  1 -eq $is_pkg_enable ] && [ 1 -eq $is_homes ]; then
			${CSTN_START_SCRIPT} start > /dev/null 2>&1;
			touch "${CSTN_RELOAD_USERDB_CACHE_MARK}"
		fi
    	;;
    *)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
    ;;
esac

